/*
 * Copyright (C) 2001-2001 OSS Nokalva Inc.  All rights reserved.
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */


/*
 * This example demonstrates how you can utilize an information object set
 * generated by the compiler to determine the type of the encoded sub-PDU
 * value carried by a nested open type field.  It also shows how you can use
 * this type information extracted from the information object set for manual
 * decoding of the encoded sub-PDU carried by the open type field.
 *
 * To run the program say:
 *
 * asn1pjav open.asn
 * cd open
 * open.bat javac
 * cd ..
 * javac -g *.java
 * java Tinfoobject
 */

/**
    Application program Tinfoobject.java demonstrates how you can utilize
    the information object set generated by the compiler to determine the
    type of the value carried by a nested open type field.  It also shows
    how you can use this type information extracted from the information
    object set for manual decoding of the nested open type field.

    The example uses the abstract syntax:

Test DEFINITIONS ::= BEGIN

    OPERATION ::= CLASS {
	  &code INTEGER UNIQUE,
	  &Type
    }

    A ::= SEQUENCE {
	  opcode OPERATION.&code ({ASet}),
	  argument OPERATION.&Type ({ASet}{@opcode})
    }


    ASet OPERATION ::= { { &code 1, &Type INTEGER } |
			 { &code 2, &Type BOOLEAN } |
			 { &code 3, &Type SEQUENCE { a INTEGER }}
		   }

END

    The example does the following:

    1. Initializes the project and creates a BER coder
    2. Creates a PDU of type A with opcode 1 and argument INTEGER : 100
    3. Encodes the PDU with automatic encoding of open types enabled.
    4. Decodes the PDU with automatic decoding of open types disabled.
       Manualy decodes the 'argument' open type field which was left
       undecoded after decoding the PDU with automatic decoding disabled.
       Does a lookup of the generated information object set to detemine
       the type of the value carried by the 'argument' open type field.
       Gets the type of the value in the 'argument' from the information
       object found as a result of the lookup.
*/

/* Compiler-generated classes */
import open.*;
import open.test.*;

/* Universal classes from the OSS runtime library */
import com.oss.asn1.*;
import com.oss.util.*;

/* Java I/O classes */
import java.io.*;

public class Tinfoobject {

    public static void main(String args[]) {

		//
		// Initialize the project.
		//
	try {
	    Open.initialize();
    	}
    	catch (Exception e) {
    	    System.out.println("Initialization failed with exception: " + e);
    	    System.exit(1);
    	}

	try {
		//
		// Run the sample.
		//
	    run();
	} finally {
		//
		// Deinitialize on program exit.
		//
	    Open.deinitialize();
	}
    }

    public static void run() {

		//
		// Instantiate and configure the coder.
		//
	Coder coder = Open.getBERCoder();

	coder.enableEncoderDebugging();
	coder.enableDecoderDebugging();
	coder.enableAutomaticEncoding();

		//
		// Create the unencoded PDU object for encoding.
		//
	A pdu = new A(1, new OpenType(new INTEGER(100)));

		//
		// Print the input to the encoder.
		//
	System.out.println("\n\tUnencoded PDU object...\n");
	System.out.println(pdu);

	ByteArrayOutputStream sink = new ByteArrayOutputStream();
	ByteArrayInputStream source = null;
	byte[] encoding = null;

	try {
		//
		// Encode the PDU.
		//
	    System.out.println("\nEncoding the PDU...\n");
	    coder.encode(pdu, sink);

		//
		// Extract the encoded PDU from the sink stream.
		//
	    encoding = sink.toByteArray();

		//
		// Print the encoding using the HexTool utility.
		//
	    System.out.println("\n\tEncoded PDU...\n");
	    HexTool.printHex(encoding);
	}
	catch (EncodeFailedException e) {
	    System.out.println("Encoding failed with exception: " + e);
	}
	catch (EncodeNotSupportedException e) {
	    System.out.println("Encoding failed with exception: " + e);
	}

	if (encoding == null)
		//
		// Encoding failed.
		//
	    return;

		//
		// Define the variable to receive the decoded PDU object.
		//
	A decodedData = null;

	try {
	    source = new ByteArrayInputStream(encoding);
		//
		// Disable automatic decoding.
		//
	    coder.disableAutomaticDecoding();

		//
		// Decode the PDU.
		//
	    System.out.println(
		"\nDecoding the PDU with automatic decoding turned off...\n");

	    decodedData = (A) coder.decode(source, new A());

	    System.out.println("\n\tDecoded PDU...\n");
	    System.out.println(decodedData);
	}
	catch (DecodeFailedException e) {
	    System.out.println("Decoding failed with exception: " + e);
	}
	catch (DecodeNotSupportedException e) {
	    System.out.println("Decoding failed with exception: " + e);
	}

	if (decodedData == null)
		//
		// Decoding failed.
		//
	    return;

	try {
		//
		// Manually decode the open type field.  The component
		// relation constraint imposed on the 'argument' indicates
		// that the type of the value carried by the open type can
		// be determined from the ASet information object set. The
		// compiler generates the information object set of the class
		// defined in the ASN.1 module.  For this example, the ASet
		// information object set is generated in the open.test.Test
		// class.  Before doing manual decoding we need to determine
		// the actual type of the 'argument'.  To determine the type,
		// we will search the ASet information object set defined in
		// the open.test.Test class for the information object where
		// the '&code' field matches the 'opcode' field of the decoded
		// PDU.
		//
	    InfoObjectSet ios = Test.aSet;
	    int n_objects = ios.getSize();
		//
		// Get the key for the search.
		//
	    long opcode = decodedData.getOpcode();
		//
		// Define a type variable.
		//
	    ASN1Type type = null;
	    for (int i = 0; i < n_objects; i++) {
		OPERATION operation = (OPERATION)ios.getElement(i);
		if (operation.getCode() == opcode) {
			//
			// Matching information object is found. Get the
			// type of the 'argument' from the matching information
			// object.
			//
		    type = operation.getType();
			//
			// Since the '&code' field of the OPERATION information
			// object class is defined as 'UNIQUE', there is no
			// need to continue the search for additional matches.
			//
		    break;
		}
	    }

	    if (type == null) {
		System.out.println(
		    "\nCannot determine the type of the 'argument'.");
		return;
	    }

		//
		// Type of the 'argument' field was determined. Do manual
		// decoding of the open type. The ASN1Type object,
		// extracted from the information object, can be passed as
		// the second argument to the decoder.
		//
	    System.out.println("Decoding the argument value...\n");
	    encoding = decodedData.getArgument().getEncodedValue();
	    source = new ByteArrayInputStream(encoding);
	    decodedData.getArgument().setDecodedValue(
		    coder.decode(source, type));
	    System.out.println("\n\tDecoded PDU...\n");
	    System.out.println(decodedData);
	}
	catch (DecodeFailedException e) {
	    System.out.println("Decoding failed with exception: " + e);
	}
	catch (DecodeNotSupportedException e) {
	    System.out.println("Decoding failed with exception: " + e);
	}
    }
}
